/*
win-x-move - easier window move/resize for ms windows.
Copyright (C) 2007 John Tsiombikas <nuclear@siggraph.org>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/
#include "gui.h"
#include "events.h"
#include "resource.h"

HWND win_main, bn_ok, bn_cancel, bn_quit, chk_enable;
int trayfied = 1;

void create_gui(void) {
	reg_win_class("winxmove_gui", win_proc);

	win_main = create_window("win-x-move", 180, 140, "winxmove_gui");
	chk_enable = create_checkbox("Enable move/resize", 5, 10, 160, 20, win_main);
	bn_ok = create_button("Ok", 5, 45, 80, 24, win_main);
	bn_cancel = create_button("Cancel", 90, 45, 80, 24, win_main);
	bn_quit = create_button("Kill win-x-move", 20, 85, 130, 24, win_main);

	SendMessage(chk_enable, BM_SETCHECK, BST_CHECKED, 0);

	tray(1);
}

void tray(int trayfy) {
	NOTIFYICONDATA nicon;
	memset(&nicon, 0, sizeof nicon);
	nicon.cbSize = sizeof nicon;
	nicon.hWnd = win_main;
	nicon.uID = 0;
	nicon.uCallbackMessage = WM_USER;
	nicon.uFlags = NIF_ICON | NIF_TIP | NIF_MESSAGE;
	nicon.hIcon = LoadIcon(GetModuleHandle(0), MAKEINTRESOURCE(ICON_WXM));
	strcpy(nicon.szTip, "Click to open the config dialog");

	if(trayfy) {
		ShowWindow(win_main, 0);
        Shell_NotifyIcon(NIM_ADD, &nicon);
	} else {
		Shell_NotifyIcon(NIM_DELETE, &nicon);
		ShowWindow(win_main, 1);
		SetActiveWindow(win_main);
	}
}

HWND create_window(const char *title, int xsz, int ysz, const char *class_name)
{
	int x = (GetSystemMetrics(SM_CXSCREEN) - xsz) / 2;
	int y = (GetSystemMetrics(SM_CYSCREEN) - ysz) / 2;
	unsigned long style = WS_OVERLAPPED | WS_CAPTION | WS_SYSMENU;
	return CreateWindow(class_name, title, style, x, y, xsz, ysz, 0, 0, GetModuleHandle(0), 0);
}

HWND create_button(const char *title, int x, int y, int xsz, int ysz, HWND parent)
{
	unsigned long style = BS_PUSHBUTTON | WS_CHILD | WS_VISIBLE | WS_TABSTOP;
	return CreateWindow("BUTTON", title, style, x, y, xsz, ysz, parent, 0, GetModuleHandle(0), 0);
}

HWND create_label(const char *text, int x, int y, int xsz, int ysz, HWND parent)
{
	unsigned long style = SS_RIGHT | WS_CHILD | WS_VISIBLE;
	return CreateWindow("STATIC", text, style, x, y, xsz, ysz, parent, 0, GetModuleHandle(0), 0);
}

HWND create_checkbox(const char *text, int x, int y, int xsz, int ysz, HWND parent)
{
	unsigned long style = BS_AUTOCHECKBOX | WS_CHILD | WS_VISIBLE | WS_TABSTOP;
	return CreateWindow("BUTTON", text, style, x, y, xsz, ysz, parent, 0, GetModuleHandle(0), 0);
}

HWND create_entry(const char *text, int x, int y, int xsz, int ysz, HWND parent)
{
	unsigned long style = ES_AUTOHSCROLL | WS_CHILD | WS_BORDER | WS_TABSTOP | WS_VISIBLE;
	return CreateWindow("EDIT", text, style, x, y, xsz, ysz, parent, 0, GetModuleHandle(0), 0);
}

void en_set_text(HWND entry, const char *text)
{
	SendMessage(entry, WM_SETTEXT, 0, (long)text);
}

const char *en_get_text(HWND entry) {
	static char buf[512];
	SendMessage(entry, WM_GETTEXT, 512, (long)buf);
	return buf;
}

void reg_win_class(const char *class_name, msg_handler_t handler) {
	WNDCLASSEX wc;
	HINSTANCE pid = GetModuleHandle(0);

	memset(&wc, 0, sizeof wc);
	wc.cbSize = sizeof wc;
	wc.hbrBackground = (HBRUSH)GetStockObject(LTGRAY_BRUSH);
	wc.hCursor = LoadCursor(pid, MAKEINTRESOURCE(IDC_ARROW));
	wc.hIcon = wc.hIconSm = LoadIcon(pid, MAKEINTRESOURCE(ICON_WXM));
	wc.hInstance = pid;
	wc.lpfnWndProc = handler;
	wc.lpszClassName = class_name;
	wc.style = CS_HREDRAW | CS_VREDRAW;
	RegisterClassEx(&wc);
}
